"use strict"

var path = require("path")
var ndarray = require("ndarray")
var GifReader = require("omggif").GifReader
var ppm = require("ppm")
var pack = require("ndarray-pack")
var through = require("through")

function defaultImage(url, cb) {
  var img = new Image()
  img.onload = function() {
    var canvas = document.createElement("canvas")
    canvas.width = img.width
    canvas.height = img.height
    var context = canvas.getContext("2d")
    context.drawImage(img, 0, 0)
    var pixels = context.getImageData(0, 0, img.width, img.height)
    cb(null, ndarray(new Uint8Array(pixels.data), [img.height, img.width, 4], [4*img.width, 4, 1], 0))
  }
  img.onerror = function(err) {
    cb(err)
  }
  img.src = url
}

//Animated gif loading
function handleGIF(url, cb) {
  var xhr = new XMLHttpRequest()
  xhr.responseType = "arraybuffer"
  xhr.overrideMimeType("application/binary")
  xhr.onerror = function(err) {
    cb(err)
  }
  xhr.onload = function() {
    if(xhr.readyState !== 4) {
      return
    }
    var data = new Uint8Array(xhr.response)
    var reader
    try {
      reader = new GifReader(data)
    } catch(err) {
      cb(err)
      return
    }
    if(reader.numFrames > 0) {
      var nshape = [reader.numFrames, reader.height, reader.width, 4]
      var ndata = new Uint8Array(nshape[0] * nshape[1] * nshape[2] * nshape[3])
      var result = ndarray(ndata, nshape)
      try {
        for(var i=0; i<reader.numFrames; ++i) {
          reader.decodeAndBlitFrameRGBA(i, ndata.subarray(
            result.index(i, 0, 0, 0),
            result.index(i+1, 0, 0, 0)))
        }
      } catch(err) {
        cb(err)
        return
      }
      cb(undefined, result)
    } else {
      var nshape = [reader.height, reader.width, 4]
      var ndata = new Uint8Array(nshape[0] * nshape[1] * nshape[2])
      var result = ndarray(ndata, nshape)
      try {
        reader.decodeAndBlitFrameRGBA(0, ndata)
      } catch(err) {
        cb(err)
        return
      }
      cb(undefined, result)
    }
  }
  xhr.open("GET", url, true)
  xhr.send()
}

//PPM loading
function handlePPM(url, cb) {
  var xhr = new XMLHttpRequest()
  xhr.responseType = "arraybuffer"
  xhr.overrideMimeType("application/binary")
  xhr.onerror = function(err) {
    cb(err)
  }
  xhr.onload = function() {
    if(xhr.readyState !== 4) {
      return
    }
    var fakeStream = through()
    ppm.parse(fakeStream, function(err, pixels) {
      if(err) {
        cb(err)
        return
      }
      var nshape = [ pixels.length, pixels[0].length, pixels[0][0].length ]
      var data = new Uint8Array(nshape[0] * nshape[1] * nshape[2])
      var result = ndarray(data, nshape)
      pack(pixels, result)
      cb(undefined, result)
    })
    fakeStream.end(new Uint8Array(xhr.response))
  }
  xhr.open("GET", url, true)
  xhr.send()
}

module.exports = function getPixels(url, cb) {
  var ext = path.extname(url)
  switch(ext.toUpperCase()) {
    case ".GIF":
      handleGIF(url, cb)
    break
    case ".PPM":
      handlePPM(url, cb)
    break
    default:
      defaultImage(url, cb)
  }
}
